﻿#light
open System
open System.Text // for Encoding.
open System.Net  // for WebClient class, HttpWebRequest class. accessible by default.
open System.IO   // for StreamReader.
open System.Web  // must add Reference to System.Web.dll -- not accessible by default.

printfn "\nBegin F# HTTP request-response test of MiniCalc Web App\n"
//let url = "http://localhost:15900/MiniCalc/Default.aspx" // note port #
let url : string = "http://localhost:15900/MiniCalc/Default.aspx"
printfn "URL under test = %s \n" url

// Get ViewState and EventValidation - define before calling
//let getVSandEV (url : string) : (string * string)  =
let getVSandEV (url : string) =
  let wc = new WebClient()
  let st = wc.OpenRead(url)
  let sr = new StreamReader(st)
  let res = sr.ReadToEnd()
  sr.Close()
  st.Close()

  let startI = res.IndexOf("id=\"__VIEWSTATE\"", 0) + 24 // location beginning of ViewState 
  let endI = res.IndexOf("\"", startI)                    // location of terminating " char
  let viewState = res.Substring(startI, endI - startI)

  let startI = res.IndexOf("id=\"__EVENTVALIDATION\"", 0) + 30 // location beginning of EventValidation 
  let endI = res.IndexOf("\"", startI)                         // location of terminating " char
  let eventValidation = res.Substring(startI, endI - startI)
  (viewState, eventValidation)

(*
// immutable array approach
let testCases = [| "001,TextBox1=5&TextBox2=3&Operation=RadioButton1&Button1=clicked,value=\"8.0000\",Add 5 and 3"
                   "002,TextBox1=5&TextBox2=3&Operation=RadioButton2&Button1=clicked,value=\"15.0000\",Multiply 5 and 3"
                   "003,TextBox1=0&TextBox2=0&Operation=RadioButton1&Button1=clicked,value=\"0.0000\",Add 0 and 0"
                |]
*)

(*
// mutable array
let testCases = Array.create 3 ""
testCases.[0] <- "001,TextBox1=5&TextBox2=3&Operation=RadioButton1&Button1=clicked,value=\"8.0000\",Add 5 and 3"
testCases.[1] <- "002,TextBox1=5&TextBox2=3&Operation=RadioButton2&Button1=clicked,value=\"15.0000\",Multiply 5 and 3"
testCases.[2] <- "003,TextBox1=0&TextBox2=0&Operation=RadioButton1&Button1=clicked,value=\"0.0000\",Add 0 and 0"
*)

(*
// F# List 
let testCases =
  [ "001,TextBox1=5&TextBox2=3&Operation=RadioButton1&Button1=clicked,value=\"8.0000\",Add 5 and 3"
    "002,TextBox1=5&TextBox2=3&Operation=RadioButton2&Button1=clicked,value=\"15.0000\",Multiply 5 and 3"
    "003,TextBox1=0&TextBox2=0&Operation=RadioButton1&Button1=clicked,value=\"0.0000\",Add 0 and 0"
  ]
*)
  
// F# array
let testCases =
  [| "001,TextBox1=5&TextBox2=3&Operation=RadioButton1" +
     "&Button1=clicked,value=\"8.0000\",Add 5 and 3"
     "002,TextBox1=5&TextBox2=3&Operation=RadioButton2" +
     "&Button1=clicked,value=\"15.0000\",Multiply 5 and 3"
     "003,TextBox1=0&TextBox2=0&Operation=RadioButton1" +
     "&Button1=clicked,value=\"0.0000\",Add 0 and 0"
  |]

// there are many ways to process a collection:                  
//let list = ["Hello";"Goodbye"]
//let x = list.Map(fun (s : string) -> s.ToUpper()) // depricated
//let result = list |> List.map (fun (s : string) -> s.ToUpper())
//list |> List.map(fun (s : string) -> s.ToUpper()) |> ignore
//let _ = list |> List.map(fun (s : string) -> s.ToUpper())
  
try
  let numPass = ref 0
  let numFail = ref 0
      
  //testCases |> Seq.for_all(fun (testCase : string) -> // for_all tests if each element meets a predicate
  testCases |>
  Seq.iter(fun (testCase : string) ->
   
  //let mutable numPass = 0 // this allows incrementing
  //let mutable numFail = 0 // . . . but now cannot print them outside the closure
    
  let delimits = [|',';'~'|];
  let tokens = testCase.Split(delimits)
  let caseID = tokens.[0]
  let input = tokens.[1]
  let expected = tokens.[2]
  let comment = tokens.[3]
  let (vs, ev) = getVSandEV url
  let data = input + "&__VIEWSTATE=" + HttpUtility.UrlEncode(vs) + "&__EVENTVALIDATION=" + HttpUtility.UrlEncode(ev)
  let buffer = Encoding.ASCII.GetBytes(data)

  // Create request object                  
  let req = WebRequest.Create(url) :?> HttpWebRequest
  req.Method <- "POST"
  req.ContentType <- "application/x-www-form-urlencoded"
  req.ContentLength <- int64 buffer.Length
  //req.ContentLength <- (int64) buffer.Length 
 
  // send HTTP request
  let reqSt = req.GetRequestStream()
  reqSt.Write (buffer, 0, buffer.Length)
  reqSt.Flush()
  reqSt.Close()
                           
  // get HTML response
  let res = req.GetResponse() :?> HttpWebResponse
  let resSt = res.GetResponseStream()
  let sr = new StreamReader(resSt)
  let html = sr.ReadToEnd()

  // examine response for expected value
  printfn "============================"
  printfn "Test case: %s" caseID
  printfn "Comment  : %s" comment
  printfn "Input    : %s" input
  printfn "Expected : %s" expected

  if html.IndexOf(expected) >= 0 then
    printfn "Pass"
    //numPass <- numPass + 1
    numPass := !numPass + 1
  else
    printfn " ** FAIL **"
    //numFail <- numFail + 1
    numFail := !numFail + 1
   
  printfn "============================"
  
  let _ = Console.ReadLine() // keypress between test cases
  //System.Console.ReadLine() |> ignore
  System.Threading.Thread.Sleep(1000)
  )
    
  printfn "Number pass = %d" !numPass
  printfn "Number fail = %d" !numFail
     
  printfn "\nEnd F# test run"
  Console.ReadLine() |> ignore
  //let _ = Console.ReadLine() // does not work when last statement in a code block
with
  | :? System.OverflowException as e ->
    printfn "Fatal overflow exception %s" e.Message
    Console.ReadLine() |> ignore
  | :? System.Exception as e ->
    printfn "Fatal: %s" e.Message
    Console.ReadLine() |> ignore
        
// :? <type> as <ident> is a dynamic type test pattern.
// This pattern matches any value whose runtime type is the given type or a subtype of the given type.
// If present, the identifier after as is bound to the value coereced to the given type.

// end source

